#version 430 core

layout (location = 0) out vec3 FragColor;
layout (location = 1) out vec3 FragColor2;
layout (location = 2) out vec3 MotionVector;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform samplerCube cubemap_tex;
uniform vec4 WS_pos_light;
uniform vec2 near_far;

in vec4 ClipSpacePositionOut;
in vec4 PrevClipSpacePositionOut;

// location of observer
uniform vec3 camera_world;

uniform struct Light {
	vec3 color;
	vec3 direction;
} light;

uniform struct Material {
	float metallic;
	float roughness;
	//vec3 F0;
	vec3 ambient;
} material;

const float PI = 3.14f;
const float E = 2.73f;

// using the fresnel schlick approximation
// h:	half vector
// v:	view direction
// F0:	reflectance for all 3 color channels
vec3 FRESNEL(vec3 h, vec3 v, vec3 F0)
{
	float cosAlpha = dot(h, v);
	return F0 + (1.0 - F0) * pow(1.0 - cosAlpha, 5.0);
}

// m2:	the roughness to the power of 2.
// h:	the half vector
// n:	surface normal
float DISTRIBUTION(float m2, vec3 h, vec3 n)
{
	float cosAlpha2 = pow(dot(h, n), 2);
	float sinAlpha2 = 1.0 - cosAlpha2;
	float tanAlpha2 = sinAlpha2 / cosAlpha2;
	
	float D = 1.0 / (m2 * pow(cosAlpha2, 2) * pow(E, tanAlpha2 / m2));
	return D;
}

// n:	surface normal
// v:	view direction
// l:	light direction
// h:	half vector
float GEOMETRY(vec3 n, vec3 v, vec3 l, vec3 h)
{
	float G = min(1.0, min( 2 * dot(n, h) * dot(n, v) / dot(v, h), 2 * dot(n, h) * dot(n, l) / dot(v, h) ) );
	return G;
}

// n:	surface normal
// v:	view direction
// l:	light direction
// F0:	reflectance of RGB channels
// r:	roughness of surface
vec3 SPECULAR(vec3 n, vec3 v, vec3 l, vec3 F0, float r)
{
	vec3 h = normalize(v + l);
	float D = DISTRIBUTION(pow(r, 2), h, n);
	float G = GEOMETRY(n, v, l, h);
	vec3 F = FRESNEL(h, v, F0);
	// original paper says PI but most formulas I found refer to 4 in the denominator
	vec3 Rs = F * D * G / (4 * dot(n, l) * dot(n, v));
	return Rs;
}

bool lies_in_shadow()
{

	// Calculate vector from surface point to light position:
	vec3 cm_lookup_vec = vert.position_world.xyz - WS_pos_light.xyz;
	// Read depth value from cubemap shadow map:
	float d_L = texture(cubemap_tex, cm_lookup_vec).r;
	// Distance between light and fragment:
	float d_S = length(cm_lookup_vec);
	// Undo previous [0;1]-mapping. Transform to world space [near,far]:
	d_L = d_L * (near_far.y - near_far.x) + near_far.x;
	// Add a small offset to prevent shadow-acne (Adjust for your scene!)
	const float eps = 0.15;
	return d_L + eps < d_S;
}

void compute_color()
{
	
	// surface normal
	vec3 n = normalize(vert.normal_world);
	// view direction
	vec3 v = normalize(camera_world - vert.position_world);
	// light source direction
	vec3 l = normalize(-light.direction);
	
	// F0 is the base reflectivity of the surface. This is an approximation. 0.02 is the reflectivity of water.
	// This interpolates the "lowest" reflectivity (i.e. the one of water) by means of the metallic attribute of
	// the surface. Hence reflectivity of very metallic surfaces will be similar to their albedo color.
	vec3 F0 = vec3(0.02);
	F0 = mix(F0, material.ambient, material.metallic);
	
	float s = material.metallic;
	float d = 1.0 - s;
	vec3 Rs = SPECULAR(n, v, l, F0, material.roughness);
	float Iia = min(max(d, 0.2), 0.5); // ambient intensity
	vec3 Ii1 = light.color;
	vec3 Ra = material.ambient;
	vec3 Rd = material.ambient / PI;
	float d_omega = 0.0002;
	
	// only one light source
	vec3 Ir = Iia*Ra + Ii1*dot(n, l)*(s * Rs + d * Rd);
	//gl_FragColor = vec4(Ir, 1.0);
	FragColor = Ir;

	vec3 NCDPos = (ClipSpacePositionOut / ClipSpacePositionOut.w).xyz;
	vec3 PrevNCDPos = (PrevClipSpacePositionOut / PrevClipSpacePositionOut.w).xyz;
	vec4 NCDPosDiff = (ClipSpacePositionOut / ClipSpacePositionOut.w) - (PrevClipSpacePositionOut / PrevClipSpacePositionOut.w);
	vec2 motionVectorCore = NCDPosDiff.xy;
	vec3 testing = NCDPosDiff.xyz;
	MotionVector = testing;
}

void main() {
	FragColor = vec3(0);
	if (!lies_in_shadow()) 
		compute_color();

	

}